/*
 * Copyright (c) 2008 Lu, Chao-Ming (Tetralet).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *	notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *	notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "notebook.h"
// The defalut Page Name
extern gchar *page_name;
// Don't show menu if there is no locales_list in rc files
extern gboolean show_menu;
// For showing the Menu if right click on a vtebox.
extern GtkWidget *locale_menu;

extern GtkWidget *window;
extern GtkWidget *notebook;

GtkWidget *current_vtebox;

void add_page()
{
	// the component of a single page
	gint current_page;

	GtkWidget *label;
	GtkWidget *hbox;
	GtkWidget *vtebox;
	GtkWidget *scrollbar;
	pid_t pid;

	label = gtk_label_new(page_name);
	hbox = gtk_hbox_new(FALSE, 0);

	// create vtebox
	vtebox = vte_terminal_new();
	init_new_page(vtebox);
	gtk_box_pack_start(GTK_BOX(hbox), vtebox, TRUE, TRUE, 0);
	// the close page event
	g_signal_connect(G_OBJECT(vtebox), "child_exited",
					 G_CALLBACK(close_page), NULL);
	// when get focus, update `current_vtebox'
	g_signal_connect(G_OBJECT(vtebox), "grab-focus",
					 G_CALLBACK(page_get_focuse), vtebox);
#ifdef IM_PATCH
	// a dirty fix for IMs, (like gcin, xcin, oxim...)
	g_signal_connect(G_OBJECT(vtebox), "key-press-event",
					 G_CALLBACK(vtebox_key_press), NULL);
#endif
	if (show_menu)
		// show the LOCALE Selection menu
		g_signal_connect(G_OBJECT(vtebox), "button-press-event",
						 G_CALLBACK(vtebox_button_press), NULL);

	// scrollbar
	scrollbar = gtk_vscrollbar_new(vte_terminal_get_adjustment(VTE_TERMINAL(vtebox)));
	gtk_box_pack_start(GTK_BOX(hbox), scrollbar, FALSE, FALSE, 0);

	// add the new page to notebook
	current_page = gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, label);
	gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(notebook), hbox,TRUE);
	gtk_notebook_set_tab_detachable(GTK_NOTEBOOK(notebook), hbox,FALSE);
	g_object_set(gtk_widget_get_parent(label), "can-focus", FALSE, NULL);

	// show the page bar if necessary
	if (current_page == 1)
		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), TRUE);

	// Got the current page's directory
	const gchar *directory = g_getenv("PWD");
	if  (current_page > 0)
	{
		directory = g_file_read_link(g_strdup_printf("/proc/%d/cwd",
													 atoi(gtk_widget_get_name(current_vtebox))), NULL);
//		g_debug("current directory = %s (/proc/%d/cwd)", directory, atoi(gtk_widget_get_name(current_vtebox)));
	}
	pid = vte_terminal_fork_command(VTE_TERMINAL(vtebox),
									g_getenv("SHELL"), NULL, NULL,
									directory, TRUE, TRUE,TRUE);
	// set `pid' as the name of vtebox. a dirty trick.
	gtk_widget_set_name(vtebox, g_strdup_printf("%d", pid));
//	g_debug("the pid of new page = %d (%d), and name = %s\n", pid, vtebox, gtk_widget_get_name(vtebox));


	// finish!
	gtk_widget_show_all(hbox);
	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), current_page);
	gtk_window_set_focus(GTK_WINDOW(window), vtebox);
//	gtk_widget_show_all(notebook);
}

void close_page (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	gint total_page=gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));
	gint current_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
	
//	g_debug("Total Page (Notebook):%d\n", total_page);

	// if it is the last page, exit application!
	if (total_page == 1)
		gtk_main_quit();
	
	// remove current page
	gtk_notebook_remove_page(GTK_NOTEBOOK(notebook), current_page);
	
	// hide the page bar if necessary
	if (total_page == 2)
	{
		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
		// FIXME: Yes, just set it to 1 x 1, 
		// and the window will be resized to correct geometry automatically.
		gtk_window_resize(GTK_WINDOW(window), 1, 1);
	}
	if ( current_page < total_page-1 )
		gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), current_page);
}

static void page_get_focuse(GtkWidget *widget, GtkWidget *vtebox)
{
//	g_debug("Change to the %d vtebox(%d), and the pid is %s!\n", 
//			 gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)) , vtebox, gtk_widget_get_name(vtebox));
	current_vtebox = vtebox;
}

#ifdef IM_PATCH
static gboolean vtebox_key_press(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	// launch IMs to deal the inputs first
//	g_debug("Got key in the vte. Launch IMs First!\n");
//	g_debug("Got key: %s (status: %X, check: %X)\n",gdk_keyval_name(event->keyval), event->state, event->state|GDK_LOCK_MASK|GDK_MOD2_MASK);
//	return (GTK_WIDGET_REALIZED(vtebox) && gtk_im_context_filter_keypress(vtebox->pvt->im_context, event));
	return gtk_im_context_filter_keypress(VTE_TERMINAL(current_vtebox)->pvt->im_context, event);
}
#endif

gboolean vtebox_button_press(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	if (event->button == 3)
	{
		gtk_menu_popup(GTK_MENU(locale_menu), NULL, NULL, NULL, NULL, 3, (guint32)gtk_get_current_event_time());
		return TRUE;
	}
	return FALSE;
}

edit_page_label_dialog()
{
	GtkWidget *dialog, *main_hbox, *icon_vbox, *icon, *main_right_vbox,
			  *state_vbox, *title, *entry_hbox, *label, *entry, *state_bottom_hbox;
	
	/* Create the dialog window */
	dialog = gtk_dialog_new_with_buttons ("Rename the tab",
										 GTK_WINDOW(window),
										 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
										 GTK_STOCK_OK,
										 GTK_RESPONSE_OK,
										 GTK_STOCK_CANCEL,
										 GTK_RESPONSE_CANCEL,
										 NULL);
	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
	gtk_dialog_set_has_separator (GTK_DIALOG(dialog), FALSE);
	// set default response when press enter in entry.
	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);

	main_hbox = gtk_hbox_new (FALSE, 5);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), main_hbox);
	main_right_vbox = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_end (GTK_BOX(main_hbox), main_right_vbox, FALSE, FALSE, 0);

	// information icon, it should be plase at the top of iconvbox
	icon_vbox = gtk_vbox_new (FALSE, 30);
	gtk_box_pack_start (GTK_BOX(main_hbox), icon_vbox, FALSE, FALSE, 0);
	icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO,
									 GTK_ICON_SIZE_DIALOG);
	gtk_box_pack_start (GTK_BOX(icon_vbox), icon, FALSE, FALSE, 0);

	state_vbox = gtk_vbox_new (FALSE, 15);
	gtk_box_pack_start (GTK_BOX(main_hbox), state_vbox, FALSE, FALSE, 0);

	title = gtk_label_new ("Please input the Label Name of this tab:");
	gtk_box_pack_start (GTK_BOX(state_vbox), title, FALSE, FALSE, 0);

	state_bottom_hbox = gtk_hbox_new (FALSE, 3);
	gtk_box_pack_end (GTK_BOX(state_vbox), state_bottom_hbox, FALSE, FALSE, 0);

	entry_hbox = gtk_hbox_new (FALSE, 5);
	gtk_box_pack_start (GTK_BOX(state_vbox), entry_hbox, FALSE, FALSE, 10);

	label = gtk_label_new ("Label Name:");
	gtk_box_pack_start (GTK_BOX(entry_hbox), label, FALSE, FALSE, 0);

	// entry area
	entry = gtk_entry_new ();
	gtk_entry_set_text(entry, gtk_label_get_text(GTK_LABEL(gtk_notebook_get_tab_label( GTK_NOTEBOOK(notebook),
									gtk_notebook_get_nth_page (GTK_NOTEBOOK(notebook),
										gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))))));
	gtk_box_pack_start (GTK_BOX (entry_hbox), entry, TRUE, TRUE, 0);
	// set activate = TRUE when [enter] pressed in entry
	gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);

	// We'll use gtk_dialog_run() to show the whole dialog
	gtk_widget_show_all (main_hbox);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
	{
//		g_debug("Got label name: %s\n",gtk_entry_get_text(GTK_ENTRY(entry)));
		if (strlen(gtk_entry_get_text(GTK_ENTRY(entry))))
		gtk_label_set_text(GTK_LABEL(gtk_notebook_get_tab_label( GTK_NOTEBOOK(notebook),
											gtk_notebook_get_nth_page (GTK_NOTEBOOK(notebook),
												gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook))))), 
										gtk_entry_get_text(GTK_ENTRY(entry)));
		else
		gtk_label_set_text(GTK_LABEL(gtk_notebook_get_tab_label( GTK_NOTEBOOK(notebook),
											gtk_notebook_get_nth_page (GTK_NOTEBOOK(notebook),
												gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook))))),
										page_name);
	}
	gtk_widget_destroy(dialog);
}
